@@ -1,17 +1,16 @@
-# $Rev: 366 $
+# $Id: ICal.pm,v 1.61 2001/12/16 22:15:42 rbowen Exp $
 package Date::ICal;
 use strict;
 
 use vars qw($VERSION $localzone $localoffset @months @leapmonths %add_units);
-$VERSION = '1.'.(qw'$Rev: 366 $')[1];
+$VERSION = (qw'$Revision: 1.61 $')[1];
 use Carp;
 use Time::Local;
 use Date::Leapyear qw();
 use Date::ICal::Duration;
 use overload '<=>' => 'compare',
              'fallback' => 1,
-             '-' => \&subtract,
-             '+' => \&add_overload;
+             '-' => \&subtract;
 
 $localzone   = $ENV{TZ} || 0;
 $localoffset = _calc_local_offset();
@@ -24,7 +23,7 @@ Date::ICal - Perl extension for ICalendar date objects.
 
 =head1 VERSION
 
-$Revision: 366 $
+$Revision: 1.61 $
 
 =head1 SYNOPSIS
 
@@ -34,7 +33,7 @@ $Revision: 366 $
     $ical = Date::ICal->new( epoch => time );
     $ical = Date::ICal->new( year => 1964,
         month => 10, day => 16, hour => 16,
-        min => 12, sec => 47 );
+        min => 12, sec => 47, tz => '0530' );
 
     $hour = $ical->hour;
     $year = $ical->year;
@@ -42,25 +41,16 @@ $Revision: 366 $
     $ical_string = $ical->ical;
     $epoch_time = $ical->epoch;
 
-    $ical2 = $ical + $duration;
-
-(Where $duration is either a duration string, like 'P2W3DT7H9M', or a
-Date::ICal::Duration (qv) object.
-
-    $ical += 'P6DT12H';
-
-    $duration = $ical1 - $ical2;
-    $ical3 = $ical1 - $duration;
-
 =head1 DESCRIPTION
 
 Date::ICal talks the ICal date format, and is intended to be a base class for
 other date/calendar modules that know about ICal time format also.
 
+See http://dates.rcbowen.com/unified.txt for details
+
 =head1 AUTHOR
 
-Rich Bowen, and the Reefknot team. Alas, Reefknot is no more. See
-http://datetime.perl.org/ for more modern and accurate modules.
+Rich Bowen, and the Reefknot team (www.reefknot)
 
 Last touched by $Author: rbowen $
 
@@ -205,9 +195,7 @@ sub new {
     bless $self, $class;
 
     if ( exists( $args{offset} ) ) {
-        # We should complain if they're trying to set a non-UTC
-        # offset on a time that's inherently UTC.  -jv
-        if ($zflag && ($args{offset} != 0)) {
+        if ($zflag) {
             carp "Time had conflicting offset and UTC info. Using UTC"
               unless $ENV{HARNESS_ACTIVE};
           } else {
@@ -271,8 +259,8 @@ sub ical {
         # No Z on the end!
         my $julian = $self->{julian};
         my $julsec = $self->{julsec};
-        my $adjust = offset_to_seconds( $args{offset} );
-        $self->add( seconds => $adjust );
+        my $adjust = _offset_to_seconds( $args{offset} );
+        $self->add( seconds => -$adjust );
         $ical =
           sprintf( '%04d%02d%02dT%02d%02d%02d', $self->year, $self->month,
           $self->day, $self->hour, $self->minute, $self->second, );
@@ -342,17 +330,17 @@ sub epoch {
 
 #}}}
 
-#{{{ sub offset_to_seconds
+#{{{ sub _offset_to_seconds
 
-=head2 offset_to_seconds
+=head2 _offset_to_seconds
 
     $seconds_plus_or_minus = offset_to_seconds($offset);
    
-Changes -0600 to -21600. Not object method, no side-effects.
+Changes -0600 to -18000. Not object method, no side-effects.
 
 =cut
 
-sub offset_to_seconds {
+sub _offset_to_seconds {
     my $offset = shift;
 
     # Relocated from offset for re-use
@@ -377,9 +365,9 @@ sub offset_to_seconds {
 
 #}}}
 
-#{{{ sub offset_from_seconds
+#{{{ sub _offset_from_seconds
 
-=head2 offset_from_seconds
+=head2 _offset_from_seconds
 
     $seconds_plus_or_minus = offset_from_seconds($offset_in_seconds);
    
@@ -388,7 +376,7 @@ Not object method, no side-effects.
 
 =cut
 
-sub offset_from_seconds {
+sub _offset_from_seconds {
     my $secoffset  = shift;
     my $hhmmoffset = 0;
 
@@ -455,7 +443,7 @@ sub offset {
     my $newoffset = undef;
 
     if ( defined($offset) ) {    # Passed in a new value
-        $newoffset = offset_to_seconds($offset);
+        $newoffset = _offset_to_seconds($offset);
 
         unless ( defined $newoffset ) { return undef; }
 
@@ -484,7 +472,7 @@ sub offset {
 
       } else {
         if ( $self->{offset} ) {
-            $offset = offset_from_seconds( $self->{offset} );
+            $offset = _offset_from_seconds( $self->{offset} );
           } else {
             $offset = 0;
         }
@@ -645,37 +633,6 @@ sub _add {
 
 #}}}
 
-# sub add_overload {{{
-
-=head2 add_overload
-
-    $date = $date1 + $duration;
-
-Where $duration is either a duration string, or a Date::ICal::Duration
-object.
-
-    $date += 'P2DT4H7M';
-
-Adds a duration to a date object. Returns a new object, or, in the case
-of +=, modifies the existing object.
-
-=cut
-
-sub add_overload {
-    my $one = shift;
-    my $two = shift;
-
-    my $ret = $one->clone;
-
-    if ( ref $two ) {
-        $ret->add( duration => $two->as_ical );
-    } else {
-        $ret->add( duration => $two );
-    }
-
-    return $ret;
-} # }}}
-
 # sub _normalize_seconds {{{
 
 =begin internal
@@ -760,26 +717,24 @@ length of the interval between the two dates. The return value is a
 Date::ICal::Duration object (qv) and allows you to get at each of the
 individual components, or the entire duration string:
 
-    $d = $date1 - $X;
-
-Note that $X can be any of the following:
-
-If $X is another Date::ICal object (or subclass thereof) then $d will be
-a Date::ICal::Duration object.
-
+    $d = $date1 - $date2;
     $week = $d->weeks; # how many weeks apart?
     $days = $d->as_days; # How many days apart?
 
-If $X is a duration string, or a Date::ICal::Diration object, then $d
-will be an object in the same class as $date1;
+TODO: In future versions, one will be able to do one or more of the
+following:
 
     $newdate = $date - $duration; 
+    # $duration is a Date::ICalDuration object, or perhaps an ical
+    # duration string.
+
+    $newdate = $date + $duration;
+    # likewise
 
 =cut
 
 sub subtract {
     my ( $date1, $date2, $reversed ) = @_;
-    my $dur;
 
     # If the order of the arguments was reversed, overload tells us
     # about it in the third argument.
@@ -787,59 +742,17 @@ sub subtract {
         ( $date2, $date1 ) = ( $date1, $date2 );
     }
 
-    if (ref $date1 && ref $date2) {
-    # If $date1 is a Date::ICal object, and $date2 is a Duration object,
-    # then we should subtract and get a date.
-        if ((ref $date2) eq 'Date::ICal::Duration') {
-            my $seconds = $date2->as_seconds;
-            my $ret = $date1->clone;
-            $ret->add( seconds => -1 * $seconds );
-            return $ret;
+    # However, it really does not make much sense unless both objects
+    # are ... um ... objects
+# TODO: check to see if $date2 is either a D::I::Duration object, or a
+# ical duration string.
+    return undef unless ref($date1) && ref($date2);
 
-        } else {
-    # If $date2 is a Date::ICal object, or some class thereof, we should
-    # subtract and get a duration
+    # OK, we have two D::I objects
+    my $days = $date1->{julian} - $date2->{julian};
+    my $secs = $date1->{julsec} - $date2->{julsec};
 
-            my $days = $date1->{julian} - $date2->{julian};
-            my $secs = $date1->{julsec} - $date2->{julsec};
-
-            return Date::ICal::Duration->new(
-              days    => $days,
-              seconds => $secs
-            );
-        } 
-    } elsif ( ref $date1 && 
-              ( $dur = Date::ICal::Duration->new( ical => $date2 ) )
-            ) {
-    # If $date1 is a Date::ICal object, and $date2 is a duration string,
-    # we should subtract and get a date
-        return $date1 - $dur; # Is that cheating?
-
-    # Otherwise, we should call them nasty names and return undef
-    } else {
-        warn "Moron";
-        return;
-    }
-
-} # }}}
-
-# sub clone {{{
-
-=head2 clone
-
-    $copy = $date->clone;
-
-Returns a replica of the date object, including all attributes.
-
-=cut
-
-sub clone {
-    my $self = shift;
-    my $class = ref $self;
-    my %hash = %$self;
-    my $new = \%hash;
-    bless $new, $class;
-    return $new;
+    return Date::ICal::Duration->new( days => $days, seconds => $secs );
 } # }}}
 
 # sub compare {{{
@@ -1094,28 +1007,14 @@ epoch.
 
 sub days_this_year {
     my ( $d, $m, $y ) = @_;
-    my @mlist = &months($y);
-    return $mlist[$m - 1] + $d - 1;
-}    #}}}
-
-# sub day_of_week {{{
-
-=head2 day_of_week
-
-    my $day_of_week = $date->day_of_week
-
-Returns the day of week as 0..6 (0 is Sunday, 6 is Saturday).
-
-=cut
-
-sub day_of_week {
-    my $self = shift;
-    return $self->{julian} % 7;
+    my $jd = greg2jd( $y, $m, $d );
+    my $janone = greg2jd( $y, 1, 1 );
+    return $jd - $janone;
 }    #}}}
 
 # sub hour {{{
 
-=head2 hour
+=head1 hour
 
     my $hour = $date->hour
 
@@ -1132,7 +1031,7 @@ sub hour {
 
 # sub min {{{
 
-=head2 min
+=head1 min
 
     my $min = $date->min;
 
@@ -1153,7 +1052,7 @@ sub minute { return min(@_); }
 
 # sub sec {{{
 
-=head2 sec
+=head1 sec
 
     my $sec = $date->sec;
 
@@ -1201,7 +1100,7 @@ sub parsetime {
 
 # sub julian/jd #{{{
 
-=head2 julian
+=head1 julian
 
   my $jd = $date->jd;
 
@@ -1241,7 +1140,7 @@ sub _calc_local_offset {
     my $gm    = timegm(@t);
 
     my $secdiff = $gm - $local;
-    return offset_from_seconds($secdiff);
+    return _offset_from_seconds($secdiff);
 }
 
 #}}}
@@ -1258,8 +1157,6 @@ sub _calc_local_offset {
 
 =item - Fix the INTERNALS file so that it actually reflects reality
 
-=back
-
 =head1 INTERNALS
 
 Please see the file INTERNALS for discussion on the internals.
@@ -1275,7 +1172,9 @@ list of patch contributors and version-by-version notes.
 
 datetime@perl.org mailing list
 
-http://datetime.perl.org/
+http://reefknot.org/
+
+http://dates.rcbowen.com/
 
 Time::Local
 
